import React from 'react'

export type ToggleChildrenFn = {
  on: boolean,
  set: (on: boolean) => any,
  toggle: () => any,
}

export type ToggleProps = {
  children: (f: ToggleChildrenFn) => JSX.Element | null,
  default?: boolean,
}

export type ToggleState = {
  on: boolean,
}

class Toggle extends React.PureComponent<ToggleProps, ToggleState> {
  state = { on: this.props.default || false }

  set = (on: boolean) =>
    this.setState({ on })
  
  toggle = () =>
    this.setState(state => ({ on: !state.on }))
  
  render() {
    return this.props.children({
      on: this.state.on,
      set: this.set,
      toggle: this.toggle,
    })
  }
}

export default Toggle
